@@ -1,10 +1,13 @@
-set rcsId {$Id: panner.tcl,v 1.19 1998/09/24 19:14:02 jfontain Exp $}
+set rcsId {$Id: panner.tcl,v 1.20 1998/10/04 14:43:02 jfontain Exp $}
 
-class panner {}
+class panner {
+    set panner::(default,HandleSize) 8
+}
 
 proc panner::panner {this parentPath args} composite {[new frame $parentPath] $args} {
     set panner::($this,handles) {}
     set panner::($this,lastManagerSize) 0
+    set panner::($this,handleSize) $panner::(default,HandleSize)
     composite::complete $this
 }
 
@@ -14,7 +17,7 @@ proc panner::options {this} {
     # force default orientation so that children are managed geometry wise
     # force panes number for initial configuration
     return [list\
-        [list -handlesize handleSize HandleSize 8]\
+        [list -handlesize handleSize HandleSize $panner::(default,HandleSize)]\
         [list -handleplacement handlePlacement HandlePlacement 0.9 0.9]\
         [list -orient orient Orient vertical]\
         [list -panes panes Panes 2 3]\
@@ -36,7 +39,7 @@ proc panner::try {this option value} {
 }
 proc panner::set-handlesize {this value} {
     # always convert to pixels for easier use and error checking and make sure value is even
-    set $composite::($this,-handlesize) [expr {(([winfo pixels $widget::($this,path) $value]+1)/2)*2}]
+    set panner::($this,handleSize) [expr {(([winfo pixels $widget::($this,path) $value]+1)/2)*2}]
     if {$composite::($this,complete)} {
         updateHandleSize $this                                                            ;# cannot update handles before they exist
     }
@@ -219,12 +222,12 @@ proc panner::split {this handleIndex coordinate} {
         set frameEnd [expr {[lindex $box 0]+[lindex $box 2]}]
     }
     if {$coordinate>$lastCoordinate} {                                                                ;# grow frame before separator
-        incr coordinate -[expr {$composite::($this,-handlesize)/2}]                  ;# set coordinate to the resized frame boundary
+        incr coordinate -[expr {$panner::($this,handleSize)/2}]                  ;# set coordinate to the resized frame boundary
         for {set itemIndex $handleIndex} {$itemIndex<=$lastIndex} {incr itemIndex} {
             if {($itemIndex%2)==0} {
                 incr framesSize [grid ${itemName}configure $path $itemIndex -minsize]
             } else {
-                incr separatorsSize $composite::($this,-handlesize)
+                incr separatorsSize $panner::($this,handleSize)
             }
         }
         set remaining [expr {$masterSize-$coordinate-$separatorsSize}]
@@ -246,12 +249,12 @@ proc panner::split {this handleIndex coordinate} {
             }
         }
     } elseif {$coordinate<$lastCoordinate} {                                                           ;# grow frame after separator
-        incr coordinate [expr {$composite::($this,-handlesize)/2}]                   ;# set coordinate to the resized frame boundary
+        incr coordinate [expr {$panner::($this,handleSize)/2}]                       ;# set coordinate to the resized frame boundary
         for {set itemIndex $handleIndex} {$itemIndex>=0} {incr itemIndex -1} {
             if {($itemIndex%2)==0} {
                 incr framesSize [grid ${itemName}configure $path $itemIndex -minsize]
             } else {
-                incr separatorsSize $composite::($this,-handlesize)
+                incr separatorsSize $panner::($this,handleSize)
             }
         }
         set remaining [expr {$coordinate-$separatorsSize}]
@@ -276,7 +279,7 @@ proc panner::split {this handleIndex coordinate} {
 }
 
 proc panner::updateHandleSize {this} {
-    set size $composite::($this,-handlesize)
+    set size $panner::($this,handleSize)
     set path $widget::($this,path)
     set lastIndex [expr {(2*$composite::($this,-panes))-2}]
     if {[string match v* $composite::($this,-orient)]} {
@@ -317,7 +320,7 @@ proc panner::resize {this size} {
         if {($itemIndex%2)==0} {
             incr lastSize [grid ${itemName}configure $path $itemIndex -minsize]
         } else {
-            incr newSize -$composite::($this,-handlesize)
+            incr newSize -$panner::($this,handleSize)
         }
     }
     set ratio [expr {double($newSize)/$lastSize}]
