@@ -1,6 +1,6 @@
-set rcsId {$Id: stooop.tcl,v 3.63 1998/09/05 20:21:54 jfontain Exp $}
+set rcsId {$Id: stooop.tcl,v 3.64 1998/10/25 18:16:30 jfontain Exp $}
 
-package provide stooop 3.6.1
+package provide stooop 3.7
 
 catch {rename proc _proc}            ;# rename proc before it is overloaded, ignore error in case of multiple inclusion of this file
 
@@ -14,7 +14,7 @@ namespace eval ::stooop {
 
     set checkCode {}                                ;# no checking by default: use an empty instruction to avoid any performance hit
     if {[info exists ::env(STOOOPCHECKALL)]} {
-        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {}}
+        array set ::env {STOOOPCHECKPROCEDURES {} STOOOPCHECKDATA {} STOOOPCHECKOBJECTS {}}
     }
     if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
         append checkCode {::stooop::checkProcedure;}
@@ -442,26 +442,45 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
                 set interface($fullClass) {}
             }
         }
+    }
 
+    if {[info exists ::env(STOOOPCHECKPROCEDURES)]||[info exists ::env(STOOOPCHECKOBJECTS)]} {
         catch {rename ::stooop::new ::stooop::_new}                              ;# gracefully handle multiple sourcing of this file
         _proc ::stooop::new {classOrId args} {           ;# use a new new procedure instead of adding debugging code to existing one
             variable newId
-            variable fullClass
-            variable interface
-
-            if {[scan $classOrId %u dummy]==0} {                                                        ;# first argument is a class
-                set constructor ${classOrId}::[namespace tail $classOrId]                               ;# generate constructor name
-                catch {$constructor}              ;# force loading in case class is in a package so namespace commands work properly
-                set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
-                # anticipate full class name storage in original new{} in order to avoid invalid object identifier error in
-                # checkProcedure{} when member procedure is invoked from within contructor, in which case full class name would
-                # have yet to be stored.
-                set fullClass([expr {$newId+1}]) $fullName                 ;# new identifier is really incremented in original new{}
-            } else {                                                                       ;# first argument is an object identifier
-                set fullName $fullClass($classOrId)                         ;# class code, if from a package, must already be loaded
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                variable fullClass
+                variable interface
             }
-            if {[info exists interface($fullName)]} {
-                error "class $fullName with pure virtual procedures should not be instanciated"
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                variable creator
+            }
+
+            if {[info exists ::env(STOOOPCHECKPROCEDURES)]} {
+                if {[scan $classOrId %u dummy]==0} {                                                    ;# first argument is a class
+                    set constructor ${classOrId}::[namespace tail $classOrId]                           ;# generate constructor name
+                    catch {$constructor}          ;# force loading in case class is in a package so namespace commands work properly
+                    set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
+                    # anticipate full class name storage in original new{} in order to avoid invalid object identifier error in
+                    # checkProcedure{} when member procedure is invoked from within contructor, in which case full class name would
+                    # have yet to be stored.
+                    set fullClass([expr {$newId+1}]) $fullName             ;# new identifier is really incremented in original new{}
+                } else {                                                                   ;# first argument is an object identifier
+                    set fullName $fullClass($classOrId)                     ;# class code, if from a package, must already be loaded
+                }
+                if {[info exists interface($fullName)]} {
+                    error "class $fullName with pure virtual procedures should not be instanciated"
+                }
+            }
+            if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+                # keep track of procedure in which creation occured (new identifier is really incremented in original new{})
+                if {[catch {set procedure [lindex [info level -1] 0]}]} {                                   ;# no invoking procedure
+                    set creator([expr {$newId+1}]) {main script}
+                } elseif {[string length $procedure]==0} {                                          ;# invoked from a namespace body
+                    set creator([expr {$newId+1}]) [uplevel namespace current]
+                } else {                            ;# store fully qualified name, visible from creator procedure invoking procedure
+                    set creator([expr {$newId+1}]) [uplevel 2 namespace which -command $procedure]
+                }
             }
             return [uplevel ::stooop::_new $classOrId $args]
         }
@@ -616,4 +635,52 @@ if {[llength [array names ::env STOOOP*]]>0} {             ;# if one or more env
         }
         puts $traceDataChannel $text
     }
+
+    if {[info exists ::env(STOOOPCHECKOBJECTS)]} {
+        # print existing objects along with creation procedure, with optional class pattern (see the string Tcl command manual)
+        _proc ::stooop::printObjects {{pattern *}} {
+            variable fullClass
+            variable creator
+
+            foreach id [lsort -integer [array names fullClass $pattern]] {
+                puts "$fullClass($id)\($id\) created in $creator($id)"
+            }
+        }
+
+        _proc ::stooop::record {} {                                                  ;# record all existing objects for later report
+            variable fullClass
+            variable checkpointFullClass
+
+            if {[catch {set procedure [lindex [info level -1] 0]}]} {
+                set procedure {main script}
+            }
+            puts "stooop::record invoked from $procedure"
+            catch {unset checkpointFullClass}
+            array set checkpointFullClass [array get fullClass]
+        }
+
+        _proc ::stooop::report {{pattern *}} {     ;# print all new or deleted object since last record, with optional class pattern
+            variable fullClass
+            variable checkpointFullClass
+            variable creator
+
+            if {[catch {set procedure [lindex [info level -1] 0]}]} {
+                set procedure {main script}
+            }
+            puts "stooop::report invoked from $procedure:"
+            set checkpointIds [lsort -integer [array names checkpointFullClass $pattern]]
+            set currentIds [lsort -integer [array names fullClass $pattern]]
+            foreach id $currentIds {
+                if {[lsearch -exact $checkpointIds $id]<0} {
+                    puts "+ $fullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+            foreach id $checkpointIds {
+                if {[lsearch -exact $currentIds $id]<0} {
+                    puts "- $checkpointFullClass($id)\($id\) created in $creator($id)"
+                }
+            }
+        }
+    }
+
 }
